@@ -1,24 +1,22 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2009-2017 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2009-2021 -- leonerd@leonerd.org.uk
 
-package Tickit::Widget;
+use Object::Pad 0.32;
 
-use strict;
-use warnings;
-
-our $VERSION = '0.53';
+package Tickit::Widget 0.54;
+class Tickit::Widget :repr(HASH);
 
 use Carp;
-use Scalar::Util qw( weaken );
+use Scalar::Util qw( blessed weaken );
 use List::Util 1.33 qw( all );
 
 use Tickit::Pen;
 use Tickit::Style;
 use Tickit::Utils qw( textwidth );
 use Tickit::Window 0.57;  # $win->bind_event
-use Tickit::Event 0.63;  # $info->type("newapi") on Focus
+use Tickit::Event 0.66;  # $info->type newapi
 
 use constant PEN_ATTR_MAP => { map { $_ => 1 } @Tickit::Pen::ALL_ATTRS };
 
@@ -79,7 +77,9 @@ order.
 
 =cut
 
-=head2 $widget = Tickit::Widget->new( %args )
+=head2 new
+
+   $widget = Tickit::Widget->new( %args )
 
 Constructs a new C<Tickit::Widget> object. Must be called on a subclass that
 implements the required methods; see the B<SUBCLASS METHODS> section below.
@@ -105,61 +105,72 @@ of the style classes or the default definitions.
 The hash should contain style keys, optionally suffixed by style tags, giving
 values.
 
- style => {
-   'fg'        => 3,
-   'fg:active' => 5,
- }
+   style => {
+     'fg'        => 3,
+     'fg:active' => 5,
+   }
 
 =back
 
 =cut
 
-sub new
+has @_style_classes;
+has $_style_direct;
+has %_style_tag;
+
+BUILD
 {
-   my $class = shift;
    my %args = @_;
 
+   my $class = ref $self;
    foreach my $method (qw( lines cols render_to_rb )) {
       $class->can( $method ) or
          croak "$class cannot ->$method - do you subclass and implement it?";
    }
 
-   my $self = bless {
-      classes => delete $args{classes} // [ delete $args{class} ],
-   }, $class;
+   @_style_classes = @{ delete $args{classes} // [ delete $args{class} ] };
 
    # Legacy direct-applied-style argument support
    $args{$_} and $args{style}{$_} = delete $args{$_} for @Tickit::Pen::ALL_ATTRS;
 
    if( my $style = delete $args{style} ) {
-      my $tagset = $self->{style_direct} = Tickit::Style::_Tagset->new;
+      my $tagset = $_style_direct = Tickit::Style::_Tagset->new;
       foreach my $key ( keys %$style ) {
          $tagset->add( $key, $style->{$key} );
       }
    }
 
    $self->_update_pen( $self->get_style_pen );
-
-   return $self;
 }
 
+has $_parent :reader;
+has $_window :reader;
+has $_pen    :reader;
+
+has $_focus_pending;
+
+has %_event_ids;
+
 =head1 METHODS
 
 =cut
 
-=head2 @classes = $widget->style_classes
+=head2 style_classes
+
+   @classes = $widget->style_classes
 
 Returns a list of the style class names this Widget has.
 
 =cut
 
-sub style_classes
+method style_classes
 {
-   my $self = shift;
-   return @{ $self->{classes} };
+   return @_style_classes;
 }
 
-=head2 $widget->set_style_tag( $tag, $value )
+=head2 set_style_tag
+
+   $widget->set_style_tag( $tag, $value )
 
 Sets the (boolean) state of the named style tag. After calling this method,
 the C<get_style_*> methods may return different results. No resizing or
@@ -175,19 +186,18 @@ changes and decide for itself.
 my %KEYS_BY_TYPE_CLASS_TAG;
 Tickit::Style::on_style_load( sub { undef %KEYS_BY_TYPE_CLASS_TAG } );
 
-sub set_style_tag
+method set_style_tag
 {
-   my $self = shift;
    my ( $tag, $value ) = @_;
 
    # Early-return on no change
-   return if !$self->{style_tag}{$tag} == !$value;
+   return if !$_style_tag{$tag} == !$value;
 
    # Work out what style keys might depend on this tag
    my %values;
 
-   if( $self->{style_direct} ) {
-      KEYSET: foreach my $keyset ( $self->{style_direct}->keysets ) {
+   if( $_style_direct ) {
+      KEYSET: foreach my $keyset ( $_style_direct->keysets ) {
          $keyset->tags->{$tag} or next KEYSET;
 
          $values{$_} ||= [] for keys %{ $keyset->style };
@@ -217,21 +227,21 @@ sub set_style_tag
    my @old_values = $self->get_style_values( @keys );
    $values{$keys[$_]}[0] = $old_values[$_] for 0 .. $#keys;
 
-   $self->{style_tag}{$tag} = !!$value;
+   $_style_tag{$tag} = !!$value;
 
    $self->_style_changed_values( \%values );
 }
 
-sub _style_tags
+method _style_tags
 {
-   my $self = shift;
-   my $tags = $self->{style_tag};
-   return join "|", sort grep { $tags->{$_} } keys %$tags;
+   return join "|", sort grep { $_style_tag{$_} } keys %_style_tag;
 }
 
-=head2 @values = $widget->get_style_values( @keys )
+=head2 get_style_values
 
-=head2 $value = $widget->get_style_values( $key )
+   @values = $widget->get_style_values( @keys )
+
+   $value = $widget->get_style_values( $key )
 
 Returns a list of values for the given keys of the currently-applied style.
 For more detail see the L<Tickit::Style> documentation. Returns just one value
@@ -239,9 +249,10 @@ in scalar context.
 
 =cut
 
-sub get_style_values
+has %_style_cache;
+
+method get_style_values
 {
-   my $self = shift;
    my @keys = @_;
 
    my $type = $self->_widget_style_type;
@@ -249,8 +260,7 @@ sub get_style_values
    my @set = ( 0 ) x @keys;
    my @values = ( undef ) x @keys;
 
-   my $tags = $self->{style_tag};
-   my $cache = $self->{style_cache}{$self->_style_tags} ||= {};
+   my $cache = $_style_cache{$self->_style_tags} ||= {};
 
    foreach my $i ( 0 .. $#keys ) {
       next unless exists $cache->{$keys[$i]};
@@ -260,7 +270,7 @@ sub get_style_values
    }
 
    my @classes = ( $self->style_classes, undef );
-   my $tagset = $self->{style_direct};
+   my $tagset = $_style_direct;
 
    while( !all { $_ } @set and @classes ) {
       # First time around this uses the direct style, if set. Thereafter uses
@@ -268,7 +278,7 @@ sub get_style_values
       defined $tagset or $tagset = Tickit::Style::_ref_tagset( $type, shift @classes );
 
       KEYSET: foreach my $keyset ( $tagset->keysets ) {
-         $tags->{$_} or next KEYSET for keys %{ $keyset->tags };
+         $_style_tag{$_} or next KEYSET for keys %{ $keyset->tags };
 
          my $style = $keyset->style;
 
@@ -282,6 +292,12 @@ sub get_style_values
       }
 
       undef $tagset;
+
+      # After all the classes, try again with type as "*"
+      if( $type ne "*" and !@classes ) {
+         $type = "*";
+         @classes = ( $self->style_classes, undef );
+      }
    }
 
    foreach my $i ( 0 .. $#keys ) {
@@ -294,23 +310,26 @@ sub get_style_values
    return $values[0];
 }
 
-=head2 $pen = $widget->get_style_pen( $prefix )
+=head2 get_style_pen
+
+   $pen = $widget->get_style_pen( $prefix )
 
 A shortcut to calling C<get_style_values> to collect up the pen attributes,
 and form a L<Tickit::Pen::Immutable> object from them. If C<$prefix> is
 supplied, it will be prefixed on the pen attribute names with an underscore
-(which would be read from the stylesheet file as a hypen). Note that the
+(which would be read from the stylesheet file as a hyphen). Note that the
 returned pen instance is immutable, and may be cached.
 
 =cut
 
-sub get_style_pen
+has %_style_pen_cache;
+
+method get_style_pen
 {
-   my $self = shift;
    my $class = ref $self;
    my ( $prefix ) = @_;
 
-   return $self->{style_pen_cache}{$self->_style_tags}{$prefix//""} ||= do {
+   return $_style_pen_cache{$self->_style_tags}{$prefix//""} ||= do {
       my @keys = map { defined $prefix ? "${prefix}_$_" : $_ } @Tickit::Pen::ALL_ATTRS;
 
       my %attrs;
@@ -320,21 +339,24 @@ sub get_style_pen
    };
 }
 
-=head2 $text = $widget->get_style_text
+=head2 get_style_text
+
+   $text = $widget->get_style_text
 
 A shortcut to calling C<get_style_values> for a single key called C<"text">.
 
 =cut
 
-sub get_style_text
+method get_style_text
 {
-   my $self = shift;
    my $class = ref $self;
 
    return $self->get_style_values( "text" ) // croak "$class style does not define text";
 }
 
-=head2 $widget->set_style( %defs )
+=head2 set_style
+
+   $widget->set_style( %defs )
 
 Changes the widget's direct-applied style.
 
@@ -351,9 +373,8 @@ applied style tags does not invalidate the caches.
 
 =cut
 
-sub set_style
+method set_style
 {
-   my $self = shift;
    my %defs = @_;
 
    my $new = Tickit::Style::_Tagset->new;
@@ -369,25 +390,24 @@ sub set_style
    my @old_values = $self->get_style_values( @keys );
    $values{$keys[$_]}[0] = $old_values[$_] for 0 .. $#keys;
 
-   if( $self->{style_direct} ) {
-      $self->{style_direct}->merge( $new );
+   if( $_style_direct ) {
+      $_style_direct->merge( $new );
    }
    else {
-      $self->{style_direct} = $new;
+      $_style_direct = $new;
    }
 
    $self->_style_changed_values( \%values, 1 );
 }
 
-sub _style_changed_values
+method _style_changed_values
 {
-   my $self = shift;
    my ( $values, $invalidate_caches ) = @_;
 
    my @keys = keys %$values;
 
    if( $invalidate_caches ) {
-      foreach my $keyset ( values %{ $self->{style_cache} } ) {
+      foreach my $keyset ( values %_style_cache ) {
          delete $keyset->{$_} for @keys;
       }
    }
@@ -416,7 +436,7 @@ sub _style_changed_values
    }
 
    if( $invalidate_caches ) {
-      foreach my $penset ( values %{ $self->{style_pen_cache} } ) {
+      foreach my $penset ( values %_style_pen_cache ) {
          delete $penset->{$_} for keys %changed_pens;
       }
    }
@@ -463,7 +483,9 @@ sub _style_changed_values
    }
 }
 
-=head2 $widget->set_window( $window )
+=head2 set_window
+
+   $widget->set_window( $window )
 
 Sets the L<Tickit::Window> for the widget to draw on. Setting C<undef> removes
 the window.
@@ -478,28 +500,27 @@ This method may invoke the C<window_gained> and C<window_lost> methods.
 
 =cut
 
-sub set_window
+method set_window
 {
-   my $self = shift;
    my ( $window ) = @_;
 
    # Early out if no change
-   return if !$window and !$self->window;
-   return if $window and $self->window and $self->window == $window;
+   return if !$window and !$_window;
+   return if $window and $_window and $_window == $window;
 
-   if( $self->{window} and !$window ) {
-      $self->{window}->set_pen( undef );
-      $self->window_lost( $self->{window} );
+   if( $_window and !$window ) {
+      $_window->set_pen( undef );
+      $self->window_lost( $_window );
    }
 
-   $self->{window} = $window;
+   $_window = $window;
 
    if( $window ) {
-      $window->set_pen( $self->{pen} );
+      $window->set_pen( $_pen );
 
-      $self->window_gained( $self->{window} );
+      $self->window_gained( $_window );
 
-      $window->take_focus if delete $self->{focus_pending};
+      $window->take_focus, undef $_focus_pending if $_focus_pending;
 
       $self->reshape;
 
@@ -507,37 +528,34 @@ sub set_window
    }
 }
 
-sub window_gained
+method window_gained
 {
-   my $self = shift;
-
    my $window = $self->window;
 
    weaken $self;
 
-   my $event_ids = $self->{event_ids} //= {};
-
-   $event_ids->{geomchange} = $window->bind_event( geomchange => sub {
+   $_event_ids{geomchange} = $window->bind_event( geomchange => sub {
       $self->reshape;
       $self->redraw if !$self->parent;
    } );
 
-   $event_ids->{expose} = $window->bind_event( expose => sub {
+   $_event_ids{expose} = $window->bind_event( expose => sub {
       my ( $win, undef, $info ) = @_;
-      $win->is_visible or return;
+      $win->is_visible or return 1;
 
-      $info->rb->setpen( $self->{pen} );
+      $info->rb->setpen( $_pen );
 
       $self->render_to_rb( $info->rb, $info->rect );
+      return 1;
    });
 
-   $event_ids->{focus} = $window->bind_event( focus => sub {
+   $_event_ids{focus} = $window->bind_event( focus => sub {
       my ( $win, undef, $info ) = @_;
-      $self->_on_win_focus( $win, $info->type( "newapi" ), $info->win );
+      $self->_on_win_focus( $win, $info->type, $info->win );
    } ) if $self->can( "_widget_style_type" );
 
    if( $self->can( "on_key" ) or $self->KEYPRESSES_FROM_STYLE ) {
-      $event_ids->{key} = $window->bind_event( key => sub {
+      $_event_ids{key} = $window->bind_event( key => sub {
          my ( $win, undef, $info ) = @_;
 
          {
@@ -559,58 +577,53 @@ sub window_gained
       } );
    }
 
-   $event_ids->{mouse} = $window->bind_event( mouse => sub {
+   $_event_ids{mouse} = $window->bind_event( mouse => sub {
       my ( $win, undef, $info ) = @_;
-      $self->take_focus if $self->CAN_FOCUS and $info->button == 1 and $info->type eq "press";
+      $self->take_focus if $self->CAN_FOCUS and $info->type eq "press" and $info->button == 1;
       $self->on_mouse( $info ) if $self->can( "on_mouse" );
    } );
 }
 
-sub _on_win_focus
+method _on_win_focus
 {
-   my $self = shift;
    my ( $win, $focus ) = @_;
 
    $self->set_style_tag( focus => $focus eq "in" );
 }
 
-sub key_focus_next_after
+method key_focus_next_after
 {
-   my $self = shift;
    $self->parent and $self->parent->focus_next( after => $self );
    return 1;
 }
 
-sub key_focus_next_before
+method key_focus_next_before
 {
-   my $self = shift;
    $self->parent and $self->parent->focus_next( before => $self );
    return 1;
 }
 
-sub window_lost
+method window_lost
 {
-   my $self = shift;
-
    my $window = $self->window;
 
-   $window->unbind_event_id( $_ ) for values %{ $self->{event_ids} };
+   $window->unbind_event_id( $_ ) for values %_event_ids;
 }
 
-=head2 $window = $widget->window
+=head2 window
+
+   $window = $widget->window
 
 Returns the current window of the widget, if one has been set using
 C<set_window>.
 
 =cut
 
-sub window
-{
-   my $self = shift;
-   return $self->{window};
-}
+# generated
 
-=head2 $widget->set_parent( $parent )
+=head2 set_parent
+
+   $widget->set_parent( $parent )
 
 Sets the parent widget; pass C<undef> to remove the parent.
 
@@ -618,29 +631,28 @@ C<$parent>, if defined, must be a subclass of L<Tickit::ContainerWidget>.
 
 =cut
 
-sub set_parent
+method set_parent
 {
-   my $self = shift;
    my ( $parent ) = @_;
 
    !$parent or $parent->isa( "Tickit::ContainerWidget" ) or croak "Parent must be a ContainerWidget";
 
-   weaken( $self->{parent} = $parent );
+   weaken( $_parent = $parent );
 }
 
-=head2 $parent = $widget->parent
+=head2 parent
+
+   $parent = $widget->parent
 
 Returns the current container widget
 
 =cut
 
-sub parent
-{
-   my $self = shift;
-   return $self->{parent};
-}
+# generated accessor
 
-=head2 $widget->resized
+=head2 resized
+
+   $widget->resized
 
 Provided for subclasses to call when their size requirements have or may have
 changed. Re-calculates the size requirements by calling C<lines> and C<cols>
@@ -648,14 +660,15 @@ again, then calls C<set_requested_size>.
 
 =cut
 
-sub resized
+method resized
 {
-   my $self = shift;
    # 'scalar' just in case of odd behaviour in subclasses
    $self->set_requested_size( scalar $self->lines, scalar $self->cols );
 }
 
-=head2 $widget->set_requested_size( $lines, $cols )
+=head2 set_requested_size
+
+   $widget->set_requested_size( $lines, $cols )
 
 Provided for subclasses to call when their size requirements have or may have
 changed. Informs the parent that the widget requires a differently-sized
@@ -663,16 +676,18 @@ window if the dimensions are now different to last time.
 
 =cut
 
-sub set_requested_size
+has $_req_lines;
+has $_req_cols;
+
+method set_requested_size
 {
-   my $self = shift;
    my ( $new_lines, $new_cols ) = @_;
 
-   return if defined $self->{req_lines} and $self->{req_lines} == $new_lines and
-             defined $self->{req_cols}  and $self->{req_cols}  == $new_cols;
+   return if defined $_req_lines and $_req_lines == $new_lines and
+             defined $_req_cols  and $_req_cols  == $new_cols;
 
-   $self->{req_lines} = $new_lines;
-   $self->{req_cols}  = $new_cols;
+   $_req_lines = $new_lines;
+   $_req_cols  = $new_cols;
 
    if( $self->parent ) {
       $self->parent->child_resized( $self );
@@ -683,16 +698,22 @@ sub set_requested_size
    }
 }
 
-=head2 ( $lines, $cols ) = $widget->requested_size
+=head2 requested_size
+
+   ( $lines, $cols ) = $widget->requested_size
 
 Returns the requested size of the widget; its preferred dimensions. This
 method calls C<lines> and C<cols> and caches the result until the next call to
 C<resized>. Container widgets should use this method in preference to calling
 C<lines> and C<cols> directly.
 
-=head2 $lines = $widget->requested_lines
+=head2 requested_lines
 
-=head2 $cols  = $widget->requested_cols
+   $lines = $widget->requested_lines
+
+=head2 requested_cols
+
+   $cols  = $widget->requested_cols
 
 Returns one or other of the requested dimensions. Shortcuts for calling
 C<requested_size>. These are I<temporary> convenience methods to assist
@@ -700,18 +721,18 @@ container widgets during the transition to the new sizing model.
 
 =cut
 
-sub requested_size
+method requested_size
 {
-   my $self = shift;
-
-   return ( $self->{req_lines} //= $self->lines,
-            $self->{req_cols}  //= $self->cols );
+   return ( $_req_lines //= $self->lines,
+            $_req_cols  //= $self->cols );
 }
 
-sub requested_lines { ( shift->requested_size )[0] }
-sub requested_cols  { ( shift->requested_size )[1] }
+method requested_lines { ( $self->requested_size )[0] }
+method requested_cols  { ( $self->requested_size )[1] }
 
-=head2 $widget->redraw
+=head2 redraw
+
+   $widget->redraw
 
 Clears the widget's window then invokes the C<render> method. This should
 completely redraw the widget.
@@ -724,35 +745,29 @@ requirement to redraw, and have them all flushed in a fairly efficient manner.
 
 =cut
 
-sub redraw
+method redraw
 {
-   my $self = shift;
-
    $self->window or return;
    $self->window->expose;
 }
 
-=head2 $pen = $widget->pen
+=head2 pen
+
+   $pen = $widget->pen
 
 Returns the widget's L<Tickit::Pen>.
 
 =cut
 
-sub pen
-{
-   my $self = shift;
+# generated accessor
 
-   return $self->{pen};
-}
-
-sub _update_pen
+method _update_pen
 {
-   my $self = shift;
    my ( $newpen ) = @_;
 
-   return if $self->{pen} and $self->{pen} == $newpen;
+   return if $_pen and $_pen == $newpen;
 
-   $self->{pen} = $newpen;
+   $_pen = $newpen;
 
    if( $self->window ) {
       $self->window->set_pen( $newpen );
@@ -761,9 +776,11 @@ sub _update_pen
 }
 
 # Default empty implementation
-sub reshape { }
+method reshape { }
 
-=head2 $widget->take_focus
+=head2 take_focus
+
+   $widget->take_focus
 
 Calls C<take_focus> on the Widget's underlying Window, if present, or stores
 that the window should take focus when one is eventually set by C<set_window>.
@@ -773,17 +790,15 @@ true value.
 
 =cut
 
-sub take_focus
+method take_focus
 {
-   my $self = shift;
-
    croak ref($self) . " cannot ->take_focus" unless $self->CAN_FOCUS;
 
    if( my $win = $self->window ) {
       $win->take_focus if $win->is_visible;
    }
    else {
-      $self->{focus_pending} = 1;
+      $_focus_pending = 1;
    }
 }
 
@@ -792,7 +807,9 @@ sub take_focus
 Because this is an abstract class, the constructor must be called on a
 subclass which implements the following methods.
 
-=head2 $widget->render_to_rb( $renderbuffer, $rect )
+=head2 render_to_rb
+
+   $widget->render_to_rb( $renderbuffer, $rect )
 
 Called to redraw the widget's content to the given L<Tickit::RenderBuffer>.
 
@@ -800,42 +817,58 @@ Will be passed the clipping rectangle region to be rendered as a
 L<Tickit::Rect>. the method does not have to render any content outside of
 this region.
 
-=head2 $widget->reshape
+=head2 reshape
+
+   $widget->reshape
 
 Optional. Called after the window geometry is changed. Useful to distribute
 window change sizes to contained child widgets.
 
-=head2 $lines = $widget->lines
+=head2 lines
 
-=head2 $cols = $widget->cols
+   $lines = $widget->lines
+
+=head2 cols
+
+   $cols = $widget->cols
 
 Called to enquire on the requested window for this widget. It is possible that
 the actual allocated window may be larger, or smaller than this amount.
 
-=head2 $widget->window_gained( $window )
+=head2 window_gained
+
+   $widget->window_gained( $window )
 
 Optional. Called by C<set_window> when a window has been set for this widget.
 
-=head2 $widget->window_lost( $window )
+=head2 window_lost
+
+   $widget->window_lost( $window )
 
 Optional. Called by C<set_window> when C<undef> has been set as the window for
 this widget. The old window object is passed in.
 
-=head2 $handled = $widget->on_key( $ev )
+=head2 on_key
+
+   $handled = $widget->on_key( $ev )
 
 Optional. If provided, this method will be set as the C<on_key> callback for
 any window set on the widget. By providing this method a subclass can
 implement widgets that respond to user input. It receives the same event
 arguments structure as the underlying window C<on_key> event.
 
-=head2 $handled = $widget->on_mouse( $ev )
+=head2 on_mouse
+
+   $handled = $widget->on_mouse( $ev )
 
 Optional. If provided, this method will be set as the C<on_mouse> callback for
 any window set on the widget. By providing this method a subclass can
 implement widgets that respond to user input. If receives the same event
 arguments structure as the underlying window C<on_mouse> event.
 
-=head2 $widget->on_style_changed_values( %values )
+=head2 on_style_changed_values
+
+   $widget->on_style_changed_values( %values )
 
 Optional. If provided, this method will be called by C<set_style_tag> to
 inform the widget which style keys may have changed values, as a result of the
@@ -850,13 +883,17 @@ Most of the time this method may not be necessary as the C<style_reshape_keys>
 C<style_reshape_textwidth_keys>, and C<style_redraw_keys> declarations should
 suffice for most purposes.
 
-=head2 $widget->CAN_FOCUS
+=head2 CAN_FOCUS
+
+   $widget->CAN_FOCUS
 
 Optional, normally false. If this constant method returns a true value, the
 widget is allowed to take focus using the C<take_focus> method. It will also
 take focus automatically if it receives a mouse button 1 press event.
 
-=head2 $widget->KEYPRESSES_FROM_STYLE
+=head2 KEYPRESSES_FROM_STYLE
+
+   $widget->KEYPRESSES_FROM_STYLE
 
 Optional, normally false. If this constant method returns a true value, the
 widget will use style information to invoke named methods on keypresses. When
@@ -873,31 +910,77 @@ information.
 
 =cut
 
+=head1 UTILITY FUNCTIONS
+
+The following functions (not methods) are not exported by this package, but
+intended to be called by widget subclasses fully-qualified.
+
+=cut
+
+=head2 split_widget_opts
+
+   ( $widget, %opts ) = split_widget_opts( $arg )
+
+If C<$arg> is an object reference derived from C<Tickit::Widget>, return it
+directly with no extra options.
+
+If C<$arg> is a reference to an unblessed hash, return the C<widget> value
+from it (which must exist), followed by any other keys and values in no
+particular order.
+
+This function is intended for methods that add an entire collection of child
+widgets in a single call (for example L<Tickit::Widget::LinearBox>'s
+C<add_children>) where the caller wants to override options on specific ones.
+
+=cut
+
+sub split_widget_opts
+{
+   my ( $arg ) = @_;
+
+   my $child;
+   my %opts;
+
+   if( blessed $arg and $arg->isa( "Tickit::Widget" ) ) {
+      $child = $arg;
+   }
+   elsif( ref $arg eq "HASH" ) {
+      %opts = %$arg;
+      $child = delete $opts{child} or
+         croak "Expected to find a 'child' key";
+   }
+   else {
+      croak "Unsure what to do with argument $arg";
+   }
+
+   return ( $child, %opts );
+}
+
 =head1 EXAMPLES
 
 =head2 A Trivial "Hello, World" Widget
 
 The following is about the smallest possible C<Tickit::Widget> implementation,
-containing the bare minimum of functionallity. It displays the fixed string
+containing the bare minimum of functionality. It displays the fixed string
 "Hello, world" at the top left corner of its window.
 
- package HelloWorldWidget;
- use base 'Tickit::Widget';
+   package HelloWorldWidget;
+   use base 'Tickit::Widget';
 
- sub lines {  1 }
- sub cols  { 12 }
+   sub lines {  1 }
+   sub cols  { 12 }
 
- sub render_to_rb
- {
-    my $self = shift;
-    my ( $rb, $rect ) = @_;
+   sub render_to_rb
+   {
+      my $self = shift;
+      my ( $rb, $rect ) = @_;
 
-    $rb->eraserect( $rect );
+      $rb->eraserect( $rect );
 
-    $rb->text_at( 0, 0, "Hello, world" );
- }
+      $rb->text_at( 0, 0, "Hello, world" );
+   }
 
- 1;
+   1;
 
 The C<lines> and C<cols> methods tell the container of the widget what its
 minimum size requirements are, and the C<render_to_rb> method actually draws
@@ -906,18 +989,18 @@ it to the render buffer.
 A slight improvement on this would be to obtain the size of the window, and
 position the text in the centre rather than the top left corner.
 
- sub render_to_rb
- {
-    my $self = shift;
-    my ( $rb, $rect ) = @_;
-    my $win = $self->window;
+   sub render_to_rb
+   {
+      my $self = shift;
+      my ( $rb, $rect ) = @_;
+      my $win = $self->window;
 
-    $rb->eraserect( $rect );
+      $rb->eraserect( $rect );
 
-    $rb->text_at( $win->lines - 1 ) / 2, ( $win->cols - 12 ) / 2,
-       "Hello, world"
-    );
- }
+      $rb->text_at( $win->lines - 1 ) / 2, ( $win->cols - 12 ) / 2,
+         "Hello, world"
+      );
+   }
 
 =head2 Reacting To User Input
 
@@ -925,43 +1008,43 @@ If a widget subclass provides an C<on_key> method, then this will receive
 keypress events if the widget's window has the focus. This example uses it to
 change the pen foreground colour.
 
- package ColourWidget;
- use base 'Tickit::Widget';
+   package ColourWidget;
+   use base 'Tickit::Widget';
 
- my $text = "Press 0 to 7 to change the colour of this text";
+   my $text = "Press 0 to 7 to change the colour of this text";
 
- sub lines { 1 }
- sub cols  { length $text }
+   sub lines { 1 }
+   sub cols  { length $text }
 
- sub render_to_rb
- {
-    my $self = shift;
-    my ( $rb, $rect ) = @_;
-    my $win = $self->window;
+   sub render_to_rb
+   {
+      my $self = shift;
+      my ( $rb, $rect ) = @_;
+      my $win = $self->window;
 
-    $rb->eraserect( $rect );
+      $rb->eraserect( $rect );
 
-    $rb->text_at( $win->lines - 1 ) / 2, ( $win->cols - 12 ) / 2,
-       "Hello, world"
-    );
+      $rb->text_at( $win->lines - 1 ) / 2, ( $win->cols - 12 ) / 2,
+         "Hello, world"
+      );
 
-    $win->focus( 0, 0 );
- }
+      $win->focus( 0, 0 );
+   }
 
- sub on_key
- {
-    my $self = shift;
-    my ( $args ) = @_;
+   sub on_key
+   {
+      my $self = shift;
+      my ( $args ) = @_;
 
-    if( $args->type eq "text" and $args->str =~ m/[0-7]/ ) {
-       $self->set_style( fg => $args->str );
-       return 1;
-    }
+      if( $args->type eq "text" and $args->str =~ m/[0-7]/ ) {
+         $self->set_style( fg => $args->str );
+         return 1;
+      }
 
-    return 0;
- }
+      return 0;
+   }
 
- 1;
+   1;
 
 The C<render_to_rb> method sets the focus at the window's top left corner to
 ensure that the window always has focus, so the widget will receive keypress
@@ -976,40 +1059,40 @@ Similarly, by providing an C<on_mouse> method, the widget subclass will
 receive mouse events within the window of the widget. This example saves a
 list of the last 10 mouse clicks and renders them with an C<X>.
 
- package ClickerWidget;
- use base 'Tickit::Widget';
+   package ClickerWidget;
+   use base 'Tickit::Widget';
 
- # In a real Widget this would be stored in an attribute of $self
- my @points;
+   # In a real Widget this would be stored in an attribute of $self
+   my @points;
 
- sub lines { 1 }
- sub cols  { 1 }
+   sub lines { 1 }
+   sub cols  { 1 }
 
- sub render_to_rb
- {
-    my $self = shift;
-    my ( $rb, $rect ) = @_;
+   sub render_to_rb
+   {
+      my $self = shift;
+      my ( $rb, $rect ) = @_;
 
-    $rb->eraserect( $rect );
+      $rb->eraserect( $rect );
 
-    foreach my $point ( @points ) {
-       $rb->text_at( $point->[0], $point->[1], "X" );
-    }
- }
+      foreach my $point ( @points ) {
+         $rb->text_at( $point->[0], $point->[1], "X" );
+      }
+   }
 
- sub on_mouse
- {
-    my $self = shift;
-    my ( $args ) = @_;
+   sub on_mouse
+   {
+      my $self = shift;
+      my ( $args ) = @_;
 
-    return unless $args->type eq "press" and $args->button == 1;
+      return unless $args->type eq "press" and $args->button == 1;
 
-    push @points, [ $args->line, $args->col ];
-    shift @points while @points > 10;
-    $self->redraw;
- }
+      push @points, [ $args->line, $args->col ];
+      shift @points while @points > 10;
+      $self->redraw;
+   }
 
- 1;
+   1;
 
 This time there is no need to set the window focus, because mouse events do
 not need to follow the window that's in focus; they always affect the window
