@@ -1,19 +1,17 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2012-2020 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2012-2021 -- leonerd@leonerd.org.uk
 
-package Tickit::Widget::Button;
+use Object::Pad 0.41;  # :param
 
-use strict;
-use warnings;
-use base qw( Tickit::Widget );
+package Tickit::Widget::Button 0.32;
+class Tickit::Widget::Button
+   extends Tickit::Widget;
 
 use Tickit::Style;
 use Tickit::RenderBuffer qw( LINE_SINGLE LINE_DOUBLE LINE_THICK );
 
-our $VERSION = '0.30';
-
 use Tickit::Utils qw( textwidth );
 
 use constant CAN_FOCUS => 1;
@@ -135,34 +133,29 @@ Optional. Callback function to invoke when the button is clicked.
 
 =cut
 
-sub new
+has $_label    :reader         :param = undef;
+has $_on_click :reader :writer :param = undef;
+
+has $_active;
+has $_dragging_on_self;
+
+BUILD
 {
-   my $class = shift;
    my %params = @_;
 
-   my $self = $class->SUPER::new( %params );
-
-   $self->set_label( $params{label} ) if defined $params{label};
-   $self->set_on_click( $params{on_click} ) if $params{on_click};
-
    $self->set_align ( $params{align}  // 0.5 );
    $self->set_valign( $params{valign} // 0.5 );
-
-   return $self;
 }
 
-sub lines
+method lines
 {
-   my $self = shift;
    my $has_border = ( $self->get_style_values( "linetype" ) ) ne "none";
    return 1 + 2*$has_border;
 }
 
-sub cols
+method cols
 {
-   my $self = shift;
-   my $has_border = ( $self->get_style_values( "linetype" ) ) ne "none";
-   return 4 + textwidth( $self->label ) + 2*$has_border;
+   return 4 + textwidth( $self->label ) + 2;
 }
 
 =head1 ACCESSORS
@@ -175,10 +168,7 @@ sub cols
 
 =cut
 
-sub label
-{
-   return shift->{label}
-}
+# generated accessor
 
 =head2 set_label
 
@@ -188,10 +178,9 @@ Return or set the text to display in the button area.
 
 =cut
 
-sub set_label
+method set_label
 {
-   my $self = shift;
-   ( $self->{label} ) = @_;
+   ( $_label ) = @_;
    $self->redraw;
 }
 
@@ -201,11 +190,7 @@ sub set_label
 
 =cut
 
-sub on_click
-{
-   my $self = shift;
-   return $self->{on_click};
-}
+# generated accessor
 
 =head2 set_on_click
 
@@ -217,11 +202,7 @@ Return or set the CODE reference to be called when the button area is clicked.
 
 =cut
 
-sub set_on_click
-{
-   my $self = shift;
-   ( $self->{on_click} ) = @_;
-}
+# generated accessor
 
 =head2 click
 
@@ -233,17 +214,15 @@ via other parts of code.
 
 =cut
 
-sub click
+method click
 {
-   my $self = shift;
-   $self->{on_click}->( $self );
+   $_on_click->( $self );
 }
 
 # Activation by key should "flash" the button briefly on the screen as a
 # visual feedback
-sub key_click
+method key_click
 {
-   my $self = shift;
    $self->click;
    if( my $window = $self->window ) {
       $self->set_style_tag( active => 1 );
@@ -252,11 +231,10 @@ sub key_click
    return 1;
 }
 
-sub _activate
+method _activate
 {
-   my $self = shift;
    my ( $active ) = @_;
-   $self->{active} = $active;
+   $_active = $active;
    $self->set_style_tag( active => $active );
 }
 
@@ -284,10 +262,12 @@ the button area. See also L<Tickit::WidgetRole::Alignable>.
 use Tickit::WidgetRole::Alignable name => "align",  style => "h";
 use Tickit::WidgetRole::Alignable name => "valign", style => "v";
 
-sub reshape
-{
-   my $self = shift;
+has $_label_line;
+has $_label_col;
+has $_label_end;
 
+method reshape
+{
    my $win = $self->window or return;
    my $lines = $win->lines;
    my $cols  = $win->cols;
@@ -299,16 +279,15 @@ sub reshape
    my ( $lines_before, undef, $lines_after ) = $self->_valign_allocation( 1, $lines - (2 * $has_border) );
    my ( $cols_before, undef, $cols_after ) = $self->_align_allocation( $width + 2, $cols - 2 );
 
-   $self->{label_line} = $lines_before + $has_border;
-   $self->{label_col}  = $cols_before + 2;
-   $self->{label_end}  = $cols_before + $width + 2;
+   $_label_line = $lines_before + $has_border;
+   $_label_col  = $cols_before + 2;
+   $_label_end  = $cols_before + $width + 2;
 
-   $win->cursor_at( $self->{label_line}, $self->{label_col} );
+   $win->cursor_at( $_label_line, $_label_col );
 }
 
-sub render_to_rb
+method render_to_rb
 {
-   my $self = shift;
    my ( $rb, $rect ) = @_;
 
    my $win = $self->window or return;
@@ -337,17 +316,18 @@ sub render_to_rb
       foreach my $line ( $rect->linerange( 0, $lines-1 ) ) {
          $rb->erase_at( $line, 0, $cols );
       }
+      $rb->text_at( $_label_line, 0,       "[" );
+      $rb->text_at( $_label_line, $cols-1, "]" );
    }
 
-   $rb->text_at( $self->{label_line}, $self->{label_col} - 2, $marker_left );
-   $rb->text_at( $self->{label_line}, $self->{label_end}, $marker_right );
+   $rb->text_at( $_label_line, $_label_col - 2, $marker_left );
+   $rb->text_at( $_label_line, $_label_end, $marker_right );
 
-   $rb->text_at( $self->{label_line}, $self->{label_col}, $self->label );
+   $rb->text_at( $_label_line, $_label_col, $self->label );
 }
 
-sub on_mouse
+method on_mouse
 {
-   my $self = shift;
    my ( $args ) = @_;
 
    my $type = $args->type;
@@ -359,20 +339,20 @@ sub on_mouse
       $self->_activate( 1 );
    }
    elsif( $type eq "drag_start" ) {
-      $self->{dragging_on_self} = 1;
+      $_dragging_on_self = 1;
    }
    elsif( $type eq "drag_stop" ) {
-      $self->{dragging_on_self} = 0;
+      $_dragging_on_self = 0;
    }
    elsif( $type eq "drag" ) {
       # TODO: This could be neater with an $arg->srcwin
-      $self->_activate( 1 ) if $self->{dragging_on_self} and !$self->{active};
+      $self->_activate( 1 ) if $_dragging_on_self and !$_active;
    }
    elsif( $type eq "drag_outside" ) {
-      $self->_activate( 0 ) if $self->{active};
+      $self->_activate( 0 ) if $_active;
    }
    elsif( $type eq "release" ) {
-      if( $self->{active} ) {
+      if( $_active ) {
          $self->_activate( 0 );
          $self->click;
       }
