@@ -1,18 +1,17 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2014 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011-2021 -- leonerd@leonerd.org.uk
 
-package Tickit::Widget::Entry;
+use Object::Pad 0.43;  # ADJUST
+
+package Tickit::Widget::Entry 0.32;
+class Tickit::Widget::Entry
+   extends Tickit::Widget;
 
-use strict;
-use warnings;
-use base qw( Tickit::Widget );
 use Tickit::Style;
 Tickit::Window->VERSION( '0.39' ); # expose_after_scroll default on
 
-our $VERSION = '0.28';
-
 use Tickit::Utils qw( textwidth chars2cols cols2chars substrwidth );
 
 use constant CAN_FOCUS => 1;
@@ -28,20 +27,20 @@ C<Tickit::Widget::Entry> - a widget for entering text
 
 =head1 SYNOPSIS
 
- use Tickit;
- use Tickit::Widget::Entry;
+   use Tickit;
+   use Tickit::Widget::Entry;
 
- my $entry = Tickit::Widget::Entry->new(
-    on_enter => sub {
-       my ( $self, $line ) = @_;
+   my $entry = Tickit::Widget::Entry->new(
+      on_enter => sub {
+         my ( $self, $line ) = @_;
 
-       # process $line somehow
+         # process $line somehow
 
-       $self->set_text( "" );
-    },
- );
+         $self->set_text( "" );
+      },
+   );
 
- Tickit->new( root => $entry )->run;
+   Tickit->new( root => $entry )->run;
 
 =head1 DESCRIPTION
 
@@ -96,7 +95,7 @@ Delete the entire line
 
 Delete to the start of the line
 
-=item * Ctrl-W
+=item * Ctrl-W or Ctrl-Backspace
 
 Delete one word backwards
 
@@ -152,7 +151,9 @@ Move the cursor one word right
 
 =cut
 
-=head2 $entry = Tickit::Widget::Entry->new( %args )
+=head2 new
+
+   $entry = Tickit::Widget::Entry->new( %args )
 
 Constructs a new C<Tickit::Widget::Entry> object.
 
@@ -176,23 +177,21 @@ Optional. Callback function to invoke when the C<< <Enter> >> key is pressed.
 
 =cut
 
-sub new
+has $_text          :reader :param = "";
+has $_pos_ch        :reader(position) :param(position) = 0;
+has $_scrolloffs_co = 0;
+has $_overwrite     = 0;
+has %_keybindings;
+has $_on_enter      :reader :param = undef;
+
+ADJUST
 {
-   my $class = shift;
-   my %params = @_;
+   my $textlen = length $_text;
+   $_pos_ch = $textlen if $_pos_ch > $textlen;
 
-   my $self = $class->SUPER::new( %params );
-
-   $self->{text} = defined $params{text} ? $params{text} : "";
-   $self->{pos_ch} = defined $params{position} ? $params{position} : 0;
-
-   my $textlen = length $self->{text};
-   $self->{pos_ch} = $textlen if $self->{pos_ch} > $textlen;
-
-   $self->{scrolloffs_co} = 0;
-   $self->{overwrite} = 0;
-
-   $self->{keybindings} = {
+   # TODO: It'd be nice to get this out of Object::Pad but it's a list on a
+   #   hash and thus not const
+   %_keybindings = (
       'C-a' => "key_beginning_of_line",
       'C-e' => "key_end_of_line",
       'C-k' => "key_delete_line",
@@ -202,79 +201,68 @@ sub new
       'M-b' => "key_backward_word",
       'M-f' => "key_forward_word",
 
-      'Backspace' => "key_backward_delete_char",
-      'Delete'    => "key_forward_delete_char",
-      'C-Delete'  => "key_forward_delete_word",
-      'End'       => "key_end_of_line",
-      'Enter'     => "key_enter_line",
-      'Home'      => "key_beginning_of_line",
-      'Insert'    => "key_overwrite_mode",
-      'Left'      => "key_backward_char",
-      'C-Left'    => "key_backward_word",
-      'Right'     => "key_forward_char",
-      'C-Right'   => "key_forward_word",
-   };
-
-   $self->set_on_enter( $params{on_enter} ) if defined $params{on_enter};
+      'Backspace'   => "key_backward_delete_char",
+      'C-Backspace' => "key_backward_delete_word",
+      'Delete'      => "key_forward_delete_char",
+      'C-Delete'    => "key_forward_delete_word",
+      'End'         => "key_end_of_line",
+      'Enter'       => "key_enter_line",
+      'Home'        => "key_beginning_of_line",
+      'Insert'      => "key_overwrite_mode",
+      'Left'        => "key_backward_char",
+      'C-Left'      => "key_backward_word",
+      'Right'       => "key_forward_char",
+      'C-Right'     => "key_forward_word",
+   );
 
    # Since we take keyboard input we almost certainly want to take focus here
    $self->take_focus;
-
-   return $self;
 }
 
-sub lines { 1 }
-sub cols  { 5 }
+method lines { 1 }
+method cols  { 5 }
 
-sub char2col
+method char2col
 {
-   my $self = shift;
    my ( $ch ) = @_;
 
-   return scalar chars2cols $self->{text}, $ch;
+   return scalar chars2cols $_text, $ch;
 }
 
-sub pretext_width
+method pretext_width
 {
-   my $self = shift;
-
-   return 0 if $self->{scrolloffs_co} == 0;
+   return 0 if $_scrolloffs_co == 0;
    return textwidth( $self->get_style_values( "more_left" ) );
 }
 
-sub pretext_render
+method pretext_render
 {
-   my $self = shift;
    my ( $rb ) = @_;
 
    $rb->text_at( 0, 0, $self->get_style_values( "more_left" ), $self->get_style_pen( "more" ) );
 }
 
-sub posttext_width
+method posttext_width
 {
-   my $self = shift;
-
-   return 0 if textwidth( $self->text ) <= $self->{scrolloffs_co} + $self->window->cols;
+   return 0 if textwidth( $self->text ) <= $_scrolloffs_co + $self->window->cols;
    return textwidth( $self->get_style_values( "more_right" ) );
 }
 
-sub posttext_render
+method posttext_render
 {
-   my $self = shift;
    my ( $rb ) = @_;
 
    $rb->text_at( 0, 0, $self->get_style_values( "more_right" ), $self->get_style_pen( "more" ) );
 }
 
-sub render_to_rb
+method render_to_rb
 {
-   my $self = shift;
    my ( $rb, $rect ) = @_;
 
    my $cols = $self->window->cols;
 
    if( $rect->top == 0 ) {
-      my $text = substrwidth( $self->text, $self->{scrolloffs_co}, $cols );
+      my $text = substrwidth( $self->text, $_scrolloffs_co, $cols );
 
       $rb->goto( 0, 0 );
       $rb->text( $text ) if length $text;
@@ -307,13 +295,12 @@ sub render_to_rb
    $self->reposition_cursor;
 }
 
-sub _recalculate_scroll
+method _recalculate_scroll
 {
-   my $self = shift;
    my ( $pos_ch ) = @_;
 
    my $pos_co = $self->char2col( $pos_ch );
-   my $off_co = $self->{scrolloffs_co};
+   my $off_co = $_scrolloffs_co;
 
    my $pos_x = $pos_co - $off_co;
 
@@ -334,33 +321,31 @@ sub _recalculate_scroll
       $pos_x = $pos_co - $off_co;
    }
 
-   return $off_co if $off_co != $self->{scrolloffs_co};
+   return $off_co if $off_co != $_scrolloffs_co;
    return undef;
 }
 
-sub reposition_cursor
+method reposition_cursor
 {
-   my $self = shift;
    my ( $pos_ch ) = @_;
 
    my $win = $self->window or return;
 
-   $self->{pos_ch} = $pos_ch if defined $pos_ch;
+   $_pos_ch = $pos_ch if defined $pos_ch;
 
-   my $new_scrolloffs = $self->_recalculate_scroll( $self->{pos_ch} );
+   my $new_scrolloffs = $self->_recalculate_scroll( $_pos_ch );
    if( defined $new_scrolloffs ) {
-      $self->{scrolloffs_co} = $new_scrolloffs;
+      $_scrolloffs_co = $new_scrolloffs;
       $self->redraw;
    }
 
-   my $pos_x = $self->char2col( $self->{pos_ch} ) - $self->{scrolloffs_co};
+   my $pos_x = $self->char2col( $_pos_ch ) - $_scrolloffs_co;
 
    $win->cursor_at( 0, $pos_x );
 }
 
-sub _text_spliced
+method _text_spliced
 {
-   my $self = shift;
    my ( $pos_ch, $deleted, $inserted, $at_end ) = @_;
 
    my $win = $self->window;
@@ -372,7 +357,7 @@ sub _text_spliced
    my $delta_co = $insertedlen_co - $deletedlen_co;
 
    my $pos_co = $self->char2col( $pos_ch );
-   my $pos_x  = $pos_co - $self->{scrolloffs_co};
+   my $pos_x  = $pos_co - $_scrolloffs_co;
 
    # Don't bother at all if the affected range is scrolled off the right
    return if $pos_x >= $width;
@@ -410,7 +395,7 @@ sub _text_spliced
       }
    }
 
-   if( $delta_co < 0 and $self->{scrolloffs_co} + $width < textwidth $self->text ) {
+   if( $delta_co < 0 and $_scrolloffs_co + $width < textwidth $self->text ) {
       # Add extra damage to redraw the trashed posttext marker
       my $rhs_x = -$delta_co + $self->posttext_width;
 
@@ -420,9 +405,8 @@ sub _text_spliced
    }
 }
 
-sub on_key
+method on_key
 {
-   my $self = shift;
    my ( $args ) = @_;
 
    return 0 unless $self->window->is_focused;
@@ -430,7 +414,7 @@ sub on_key
    my $type = $args->type;
    my $str  = $args->str;
 
-   if( $type eq "key" and my $code = $self->{keybindings}{$str} ) {
+   if( $type eq "key" and my $code = $_keybindings{$str} ) {
       $self->$code( $str );
       return 1;
    }
@@ -442,22 +426,20 @@ sub on_key
    return 0;
 }
 
-sub on_text
+method on_text
 {
-   my $self = shift;
    my ( $text ) = @_;
 
-   $self->text_splice( $self->{pos_ch}, $self->{overwrite} ? 1 : 0, $text );
+   $self->text_splice( $_pos_ch, $_overwrite ? 1 : 0, $text );
 }
 
-sub on_mouse
+method on_mouse
 {
-   my $self = shift;
    my ( $args ) = @_;
 
    return unless $args->type eq "press" and $args->button == 1;
 
-   my $pos_ch = scalar cols2chars $self->{text}, $args->col + $self->{scrolloffs_co};
+   my $pos_ch = scalar cols2chars $_text, $args->col + $_scrolloffs_co;
    $self->set_position( $pos_ch );
 }
 
@@ -465,17 +447,13 @@ sub on_mouse
 
 =cut
 
-=head2 $on_enter = $entry->on_enter
+=head2 on_enter
 
-=cut
+   $on_enter = $entry->on_enter
 
-sub on_enter
-{
-   my $self = shift;
-   return $self->{on_enter};
-}
+=head2 set_on_enter
 
-=head2 $entry->set_on_enter( $on_enter )
+   $entry->set_on_enter( $on_enter )
 
 Return or set the CODE reference to be called when the C<key_enter_line>
 action is invoked; usually bound to the C<Enter> key.
@@ -484,37 +462,37 @@ action is invoked; usually bound to the C<Enter> key.
 
 =cut
 
-sub set_on_enter
+# generated accessor
+
+method set_on_enter
 {
-   my $self = shift;
-   ( $self->{on_enter} ) = @_;
+   ( $_on_enter ) = @_;
 }
 
-=head2 $offset = $entry->position
+=head2 position
+
+   $offset = $entry->position
 
 Returns the current entry position, in terms of characters within the text.
 
 =cut
 
-sub position
-{
-   my $self = shift;
-   return $self->{pos_ch};
-}
+# generated accessor
 
-=head2 $entry->set_position( $position )
+=head2 set_position
+
+   $entry->set_position( $position )
 
 Set the text entry position, moving the cursor
 
 =cut
 
-sub set_position
+method set_position
 {
-   my $self = shift;
    my ( $pos_ch ) = @_;
 
    $pos_ch = 0 if $pos_ch < 0;
-   $pos_ch = length $self->{text} if $pos_ch > length $self->{text};
+   $pos_ch = length $_text if $pos_ch > length $_text;
 
    $self->reposition_cursor( $pos_ch );
 }
@@ -523,32 +501,33 @@ sub set_position
 
 =cut
 
-=head2 $entry->bind_keys( $keystr => $value, ... )
+=head2 bind_keys
+
+   $entry->bind_keys( $keystr => $value, ... )
 
 Associate methods or CODE references with keypresses. On receipt of a the key
 the method or CODE reference will be invoked, being passed the stringified key
 representation and the underlying C<Term::TermKey::Key> structure.
 
- $ret = $entry->method( $keystr, $key )
- $ret = $coderef->( $entry, $keystr, $key )
+   $ret = $entry->method( $keystr, $key )
+   $ret = $coderef->( $entry, $keystr, $key )
 
 This method takes a hash of keystring/value pairs. Binding a value of C<undef>
 will remove it.
 
 =cut
 
-sub bind_keys
+method bind_keys
 {
-   my $self = shift;
    while( @_ ) {
       my $str   = shift;
       my $value = shift;
 
       if( defined $value ) {
-         $self->{keybindings}{$str} = $value;
+         $_keybindings{$str} = $value;
       }
       else {
-         delete $self->{keybindings}{$str};
+         delete $_keybindings{$str};
       }
    }
 }
@@ -561,93 +540,95 @@ used by a program to directly manipulate the text.
 
 =cut
 
-=head2 $text = $entry->text
+=head2 text
+
+   $text = $entry->text
 
 Returns the currently entered text.
 
 =cut
 
-sub text
-{
-   my $self = shift;
-   return $self->{text};
-}
+# generated accessor
 
-=head2 $entry->set_text( $text )
+=head2 set_text
+
+   $entry->set_text( $text )
 
 Replace the text in the entry box. This completely redraws the widget's
 window. It is largely provided for initialisation; for normal edits (such as
-from keybindings), it is preferrable to use C<text_insert>, C<text_delete> or
+from keybindings), it is preferable to use C<text_insert>, C<text_delete> or
 C<text_splice>.
 
 =cut
 
-sub set_text
+method set_text
 {
-   my $self = shift;
    my ( $text ) = @_;
 
-   $self->{text} = $text;
-   $self->{pos_ch} = length $text if $self->{pos_ch} > length $text;
+   $_text = $text;
+   $_pos_ch = length $text if $_pos_ch > length $text;
 
    $self->redraw;
 }
 
-=head2 $entry->text_insert( $text, $pos_ch )
+=head2 text_insert
+
+   $entry->text_insert( $text, $pos_ch )
 
 Insert the given text at the given character position.
 
 =cut
 
-sub text_insert
+method text_insert
 {
-   my $self = shift;
    my ( $text, $pos_ch ) = @_;
 
    $self->text_splice( $pos_ch, 0, $text );
 }
 
-=head2 $deleted = $entry->text_delete( $pos_ch, $len_ch )
+=head2 text_delete
+
+   $deleted = $entry->text_delete( $pos_ch, $len_ch )
 
 Delete the given section of text. Returns the deleted text.
 
 =cut
 
-sub text_delete
+method text_delete
 {
-   my $self = shift;
    my ( $pos_ch, $len_ch ) = @_;
 
    return $self->text_splice( $pos_ch, $len_ch, "" );
 }
 
-=head2 $deleted = $entry->text_splice( $pos_ch, $len_ch, $text )
+=head2 text_splice
+
+   $deleted = $entry->text_splice( $pos_ch, $len_ch, $text )
 
 Replace the given section of text with the given replacement. Returns the
 text deleted from the section.
 
 =cut
 
-sub text_splice
+method text_splice
 {
-   my $self = shift;
    my ( $pos_ch, $len_ch, $text ) = @_;
 
    my $textlen_ch = length($text);
 
    my $delta_ch = $textlen_ch - $len_ch;
 
-   my $at_end = ( $pos_ch == length $self->{text} );
+   my $at_end = ( $pos_ch == length $_text );
 
-   my $deleted = substr( $self->{text}, $pos_ch, $len_ch, $text );
+   my $deleted = substr( $_text, $pos_ch, $len_ch, $text );
 
    my $new_pos_ch;
 
-   if( $self->{pos_ch} >= $pos_ch + $len_ch ) {
+   if( $_pos_ch >= $pos_ch + $len_ch ) {
       # Cursor after splice; move to suit
       $new_pos_ch = $self->position + $delta_ch;
    }
-   elsif( $self->{pos_ch} >= $pos_ch ) {
+   elsif( $_pos_ch >= $pos_ch ) {
       # Cursor within splice; move to end
       $new_pos_ch = $pos_ch + $textlen_ch;
    }
@@ -658,12 +639,14 @@ sub text_splice
       $self->_text_spliced( $pos_ch, $deleted, $text, $at_end );
    }
 
-   $self->reposition_cursor( $new_pos_ch ) if defined $new_pos_ch and $new_pos_ch != $self->{pos_ch};
+   $self->reposition_cursor( $new_pos_ch ) if defined $new_pos_ch and $new_pos_ch != $_pos_ch;
 
    return $deleted;
 }
 
-=head2 $pos = $entry->find_bow_forward( $initial, $else )
+=head2 find_bow_forward
+
+   $pos = $entry->find_bow_forward( $initial, $else )
 
 Search forward in the string, returning the character position of the next
 beginning of word from the initial position. If none is found, returns
@@ -671,9 +654,8 @@ C<$else>.
 
 =cut
 
-sub find_bow_forward
+method find_bow_forward
 {
-   my $self = shift;
    my ( $pos, $else ) = @_;
 
    my $posttext = substr( $self->text, $pos );
@@ -681,7 +663,9 @@ sub find_bow_forward
    return $posttext =~ m/(?<=\s)\S/ ? $pos + $-[0] : $else;
 }
 
-=head2 $pos = $entry->find_eow_forward( $initial )
+=head2 find_eow_forward
+
+   $pos = $entry->find_eow_forward( $initial )
 
 Search forward in the string, returning the character position of the next
 end of word from the initial position. If none is found, returns the length of
@@ -689,9 +673,8 @@ the string.
 
 =cut
 
-sub find_eow_forward
+method find_eow_forward
 {
-   my $self = shift;
    my ( $pos ) = @_;
 
    my $posttext = substr( $self->text, $pos );
@@ -700,7 +683,9 @@ sub find_eow_forward
    return $pos + $-[0];
 }
 
-=head2 $pos = $entry->find_bow_backward( $initial )
+=head2 find_bow_backward
+
+   $pos = $entry->find_bow_backward( $initial )
 
 Search backward in the string, returning the character position of the
 previous beginning of word from the initial position. If none is found,
@@ -708,9 +693,8 @@ returns 0.
 
 =cut
 
-sub find_bow_backward
+method find_bow_backward
 {
-   my $self = shift;
    my ( $pos ) = @_;
 
    my $pretext = substr( $self->text, 0, $pos );
@@ -718,7 +702,9 @@ sub find_bow_backward
    return $pretext =~ m/.*\s(?=\S)/ ? $+[0] : 0;
 }
 
-=head2 $pos = $entry->find_eow_backward( $initial )
+=head2 find_eow_backward
+
+   $pos = $entry->find_eow_backward( $initial )
 
 Search backward in the string, returning the character position of the
 previous end of word from the initial position. If none is found, returns
@@ -726,9 +712,8 @@ C<undef>.
 
 =cut
 
-sub find_eow_backward
+method find_eow_backward
 {
-   my $self = shift;
    my ( $pos ) = @_;
 
    my $pretext = substr( $self->text, 0, $pos + 1 ); # +1 to allow if cursor is on the space
@@ -738,121 +723,92 @@ sub find_eow_backward
 
 ## Key binding methods
 
-sub key_backward_char
+method key_backward_char
 {
-   my $self = shift;
-
-   if( $self->{pos_ch} > 0 ) {
-      $self->set_position( $self->{pos_ch} - 1 );
+   if( $_pos_ch > 0 ) {
+      $self->set_position( $_pos_ch - 1 );
    }
 }
 
-sub key_backward_delete_char
+method key_backward_delete_char
 {
-   my $self = shift;
-
-   if( $self->{pos_ch} > 0 ) {
-      $self->text_delete( $self->{pos_ch} - 1, 1 );
+   if( $_pos_ch > 0 ) {
+      $self->text_delete( $_pos_ch - 1, 1 );
    }
 }
 
-sub key_backward_delete_line
+method key_backward_delete_line
 {
-   my $self = shift;
-
-   $self->text_delete( 0, $self->{pos_ch} );
+   $self->text_delete( 0, $_pos_ch );
 }
 
-sub key_backward_delete_word
+method key_backward_delete_word
 {
-   my $self = shift;
-
-   my $bow = $self->find_bow_backward( $self->{pos_ch} );
-   $self->text_delete( $bow, $self->{pos_ch} - $bow );
+   my $bow = $self->find_bow_backward( $_pos_ch );
+   $self->text_delete( $bow, $_pos_ch - $bow );
 }
 
-sub key_backward_word
+method key_backward_word
 {
-   my $self = shift;
-
-   if( $self->{pos_ch} > 0 ) {
-      $self->set_position( $self->find_bow_backward( $self->{pos_ch} ) );
+   if( $_pos_ch > 0 ) {
+      $self->set_position( $self->find_bow_backward( $_pos_ch ) );
    }
 }
 
-sub key_beginning_of_line
+method key_beginning_of_line
 {
-   my $self = shift;
-
    $self->set_position( 0 );
 }
 
-sub key_delete_line
+method key_delete_line
 {
-   my $self = shift;
-
    $self->text_delete( 0, length $self->text );
 }
 
-sub key_end_of_line
+method key_end_of_line
 {
-   my $self = shift;
-
-   $self->set_position( length $self->{text} );
+   $self->set_position( length $_text );
 }
 
-sub key_enter_line
+method key_enter_line
 {
-   my $self = shift;
-
    my $text = $self->text;
    return unless length $text;
 
-   my $on_enter = $self->{on_enter} or return;
-   $on_enter->( $self, $text );
+   $_on_enter->( $self, $text ) if $_on_enter;
 }
 
-sub key_forward_char
+method key_forward_char
 {
-   my $self = shift;
-
-   if( $self->{pos_ch} < length $self->{text} ) {
-      $self->set_position( $self->{pos_ch} + 1 );
+   if( $_pos_ch < length $_text ) {
+      $self->set_position( $_pos_ch + 1 );
    }
 }
 
 # Renamed from readline's "delete-char" because this one doesn't have the EOF
 # behaviour if input line is empty
-sub key_forward_delete_char
+method key_forward_delete_char
 {
-   my $self = shift;
-
-   if( $self->{pos_ch} < length $self->{text} ) {
-      $self->text_delete( $self->{pos_ch}, 1 );
+   if( $_pos_ch < length $_text ) {
+      $self->text_delete( $_pos_ch, 1 );
    }
 }
 
-sub key_forward_delete_word
+method key_forward_delete_word
 {
-   my $self = shift;
-
-   my $bow = $self->find_bow_forward( $self->{pos_ch}, length $self->text );
-   $self->text_delete( $self->{pos_ch}, $bow - $self->{pos_ch} );
+   my $bow = $self->find_bow_forward( $_pos_ch, length $self->text );
+   $self->text_delete( $_pos_ch, $bow - $_pos_ch );
 }
 
-sub key_forward_word
+method key_forward_word
 {
-   my $self = shift;
-
-   my $bow = $self->find_bow_forward( $self->{pos_ch}, length $self->text );
+   my $bow = $self->find_bow_forward( $_pos_ch, length $self->text );
    $self->set_position( $bow );
 }
 
-sub key_overwrite_mode
+method key_overwrite_mode
 {
-   my $self = shift;
-
-   $self->{overwrite} = !$self->{overwrite};
+   $_overwrite = !$_overwrite;
 }
 
 =head1 TODO
