@@ -1,19 +1,18 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2020 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011-2021 -- leonerd@leonerd.org.uk
 
-package Tickit::Widget::Frame;
+use Object::Pad 0.41;  # :param
+
+package Tickit::Widget::Frame 0.34;
+class Tickit::Widget::Frame
+   extends Tickit::SingleChildWidget;
 
-use strict;
-use warnings;
-use base qw( Tickit::SingleChildWidget );
 use Tickit::Style;
 
 use Tickit::WidgetRole::Alignable name => "title_align";
 
-our $VERSION = '0.32';
-
 use Carp;
 
 use Tickit::Pen;
@@ -132,29 +131,27 @@ For more details see the accessors below.
 
 =cut
 
-sub new
+has $_title    :reader :param = undef;
+has %_has_edge;
+
+BUILD
 {
-   my $class = shift;
    my %args = @_;
 
-   my $self = $class->SUPER::new( %args );
-
-   $self->set_title( $args{title} ) if defined $args{title};
    $self->set_title_align( $args{title_align} || 0 );
 
    # Prepopulate has_* caches
    $self->on_style_changed_values;
 
-   return $self;
+   $self->set_child( $args{child} ) if $args{child};
 }
 
 =head1 ACCESSORS
 
 =cut
 
-sub on_style_changed_values
+method on_style_changed_values
 {
-   my $self = shift;
    my %values = @_;
 
    my $reshape = 0;
@@ -168,25 +165,23 @@ sub on_style_changed_values
       my $new = ( $values{"linetype_$_"}[1] // $self->get_style_values( "linetype_$_") // $linetype )
          ne "none";
 
-      $reshape = 1 if $self->{"has_$_"} != $new;
-      $self->{"has_$_"} = $new;
+      $reshape = 1 if $_has_edge{$_} != $new;
+      $_has_edge{$_} = $new;
    }
 
    $self->reshape if $reshape;
 }
 
-sub lines
+method lines
 {
-   my $self = shift;
    my $child = $self->child;
-   return ( $child ? $child->requested_lines : 0 ) + $self->{has_top} + $self->{has_bottom};
+   return ( $child ? $child->requested_lines : 0 ) + $_has_edge{top} + $_has_edge{bottom};
 }
 
-sub cols
+method cols
 {
-   my $self = shift;
    my $child = $self->child;
-   return ( $child ? $child->requested_cols : 0 ) + $self->{has_left} + $self->{has_right};
+   return ( $child ? $child->requested_cols : 0 ) + $_has_edge{left} + $_has_edge{right};
 }
 
 use constant {
@@ -220,11 +215,7 @@ my %LINESTYLES = (
 
 =cut
 
-sub title
-{
-   my $self = shift;
-   return $self->{title};
-}
+# generated accessor
 
 =head2 set_title
 
@@ -235,10 +226,9 @@ frame.
 
 =cut
 
-sub set_title
+method set_title
 {
-   my $self = shift;
-   $self->{title} = $_[0];
+   ( $_title ) = @_;
    $self->redraw;
 }
 
@@ -258,22 +248,20 @@ See also L<Tickit::WidgetRole::Alignable>.
 =cut
 
 ## This should come from Tickit::ContainerWidget
-sub children_changed { shift->reshape }
+method children_changed { $self->reshape }
 
-sub reshape
+method reshape
 {
-   my $self = shift;
-
    my $window = $self->window or return;
    my $child  = $self->child  or return;
 
    my $lines = $window->lines;
    my $cols  = $window->cols;
 
-   my $extra_lines = $self->{has_top} + $self->{has_bottom};
-   my $extra_cols  = $self->{has_left} + $self->{has_right};
+   my $extra_lines = $_has_edge{top} + $_has_edge{bottom};
+   my $extra_cols  = $_has_edge{left} + $_has_edge{right};
    if( $lines > $extra_lines and $cols > $extra_cols ) {
-      my @geom = ( $self->{has_top}, $self->{has_left}, $lines - $extra_lines, $cols - $extra_cols );
+      my @geom = ( $_has_edge{top}, $_has_edge{left}, $lines - $extra_lines, $cols - $extra_cols );
 
       if( my $childwin = $child->window ) {
          $childwin->change_geometry( @geom );
@@ -290,9 +278,8 @@ sub reshape
    }
 }
 
-sub render_to_rb
+method render_to_rb
 {
-   my $self = shift;
    my ( $rb, $rect ) = @_;
 
    $rb->setpen( $self->get_style_pen( "frame" ) );
@@ -317,8 +304,8 @@ sub render_to_rb
 
    my $h_caps = ( $left_is_line ? 0 : CAP_START ) | ( $right_is_line  ? 0 : CAP_END );
    my $v_caps = ( $top_is_line  ? 0 : CAP_START ) | ( $bottom_is_line ? 0 : CAP_END );
-   my $v_start = $top_is_line ? 0 : $self->{has_top};
-   my $v_end   = $bottom_is_line ? $bottom : $bottom - $self->{has_bottom};
+   my $v_start = $top_is_line ? 0 : $_has_edge{top};
+   my $v_end   = $bottom_is_line ? $bottom : $bottom - $_has_edge{bottom};
 
    my $linechars;
    my $style;
