@@ -1,18 +1,17 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2013-2020 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2013-2021 -- leonerd@leonerd.org.uk
 
-package Tickit::Widget::HSplit;
+use Object::Pad 0.27;
+
+package Tickit::Widget::HSplit 0.32;
+class Tickit::Widget::HSplit
+   extends Tickit::Widget::LinearSplit;
 
-use strict;
-use warnings;
-use base qw( Tickit::Widget::LinearSplit );
 use Tickit::Style;
 use Tickit::RenderBuffer qw( LINE_SINGLE CAP_BOTH );
 
-our $VERSION = '0.30';
-
 use Carp;
 
 use List::Util qw( sum max );
@@ -98,56 +97,34 @@ use constant VALUE_METHOD => "lines";
 
 Constructs a new C<Tickit::Widget::HSplit> object.
 
-Takes the following named arguments
-
-=over 8
-
-=item top_child => WIDGET
-
-=item bottom_child => WIDGET
-
-Child widgets to use.
-
-These options are now discouraged in favour of the L</set_top_child> and
-L</set_bottom_child> methods.
-
-=back
-
 =cut
 
-sub new
+BUILD
 {
-   my $class = shift;
    my %args = @_;
 
-   my $self = $class->SUPER::new( %args );
+   croak "The 'top_child' constructor argument to ${\ref $self} is no longer recognised; use ->set_top_child instead"
+      if $args{top_child};
 
-   Carp::carp( "The 'top_child' constructor argument to $class is discouraged; use ->set_top_child instead" ) and
-      $self->set_top_child   ( $args{top_child}    ) if $args{top_child};
-
-   Carp::carp( "The 'bottom_child' constructor argument to $class is discouraged; use ->set_bottom_child instead" ) and
-      $self->set_bottom_child( $args{bottom_child} ) if $args{bottom_child};
-
-   return $self;
+   croak "The 'bottom_child' constructor argument to ${\ref $self} is no longer recognised; use ->set_bottom_child instead"
+      if $args{bottom_child};
 }
 
-sub lines
+method lines
 {
-   my $self = shift;
    my $spacing = $self->get_style_values( "spacing" );
    return sum(
-      $self->{A_child} ? $self->{A_child}->requested_lines : 1,
+      $self->top_child    ? $self->top_child->requested_lines    : 1,
       $spacing,
-      $self->{B_child} ? $self->{B_child}->requested_lines : 1,
+      $self->bottom_child ? $self->bottom_child->requested_lines : 1,
    );
 }
 
-sub cols
+method cols
 {
-   my $self = shift;
    return max(
-      $self->{A_child} ? $self->{A_child}->requested_cols : 1,
-      $self->{B_child} ? $self->{B_child}->requested_cols : 1,
+      $self->top_child    ? $self->top_child->requested_cols    : 1,
+      $self->bottom_child ? $self->bottom_child->requested_cols : 1,
    );
 }
 
@@ -187,46 +164,41 @@ use as chaining mutators; e.g.
       ->set_top_child   ( Tickit::Widget::Box->new ... )
       ->set_bottom_child( Tickit::Widget::Box->new ... );
 
-These should be preferred over using the C<top_child> and C<bottom_child>
-constructor arguments, which are now discouraged.
-
 =cut
 
 *bottom_child     = __PACKAGE__->can( "B_child" );
 *set_bottom_child = __PACKAGE__->can( "set_B_child" );
 
-sub _make_child_geom
+method _make_child_geom
 {
-   my $self = shift;
    my ( $start, $len ) = @_;
    return ( $start, 0, $len, $self->window->cols );
 }
 
-sub render_to_rb
+method render_to_rb
 {
-   my $self = shift;
    my ( $rb, $rect ) = @_;
 
-   my $split_len = $self->{split_len};
+   my $split_len = $self->_split_len;
+   my $split_at  = $self->_split_at;
 
    my $cols = $self->window->cols;
 
    $rb->setpen( $self->get_style_pen( "split" ) );
 
-   $rb->hline_at( $self->{split_at}, 0, $cols-1, LINE_SINGLE, undef, CAP_BOTH );
+   $rb->hline_at( $split_at, 0, $cols-1, LINE_SINGLE, undef, CAP_BOTH );
 
    foreach my $line ( $rect->linerange( 1, $split_len-2 ) ) {
-      $rb->erase_at( $self->{split_at} + $line, 0, $cols );
+      $rb->erase_at( $split_at + $line, 0, $cols );
    }
 
    if( $split_len > 1 ) {
-      $rb->hline_at( $self->{split_at} + $split_len - 1, 0, $cols-1, LINE_SINGLE, undef, CAP_BOTH );
+      $rb->hline_at( $split_at + $split_len - 1, 0, $cols-1, LINE_SINGLE, undef, CAP_BOTH );
    }
 }
 
-sub on_mouse
+method on_mouse
 {
-   my $self = shift;
    my ( $args ) = @_;
 
    if( $args->type ne "wheel" and $args->button == 1 ) {
