@@ -1,23 +1,24 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2009-2013 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2009-2021 -- leonerd@leonerd.org.uk
 
-package Tickit::Widget::LinearBox;
+use Object::Pad 0.27;
+
+package Tickit::Widget::LinearBox 0.49;
+class Tickit::Widget::LinearBox
+   extends Tickit::ContainerWidget;
+
+use experimental 'postderef';
 
-use strict;
-use warnings;
-use base qw( Tickit::ContainerWidget );
 use Tickit::RenderBuffer;
 
-our $VERSION = '0.46';
-
 use Carp;
+use Syntax::Keyword::Dynamically;
 
 use Tickit::Utils qw( distribute );
 
 use List::Util qw( sum );
-use List::MoreUtils qw( firstidx );
 
 =head1 NAME
 
@@ -58,182 +59,214 @@ returned by C<get_child_base>.
 
 =cut
 
-=head2 $widget = Tickit::Widget::LinearBox->new( %args )
+=head2 new
+
+   $widget = Tickit::Widget::LinearBox->new( %args )
 
 Returns a new C<Tickit::Widget::LinearBox>.
 
-Takes the following named argmuents:
-
-=over 8
-
-=item children => ARRAY[Tickit::Widget]
-
-Optional. If provided, the widgets in this array will be added, with no
-additional options.
-
-=back
-
 =cut
 
-sub new
+has @_children;
+
+has $_suppress_redistribute;
+
+sub BUILDARGS
 {
    my $class = shift;
    my %args = @_;
 
    exists $args{$_} and $args{style}{$_} = delete $args{$_} for qw( spacing );
 
-   my $self = $class->SUPER::new( %args );
+   return $class->SUPER::BUILDARGS( %args );
+}
 
-   $self->{children} = [];
+BUILD
+{
+   my %args = @_;
 
-   $self->add( $_ ) for @{ $args{children} };
-
-   return $self;
+   if( $args{children} ) {
+      croak "The 'children' constructor argument to ${\ref $self} is no longer recognised; use ->add_children instead";
+   }
 }
 
 =head1 METHODS
 
 =cut
 
-=head2 @children = $widget->children
+=head2 children
+
+   @children = $widget->children
 
 In scalar context, returns the number of contained children. In list context,
 returns a list of all the child widgets.
 
 =cut
 
-sub children
+method children
 {
-   my $self = shift;
-
-   return @{ $self->{children} };
+   return @_children;
 }
 
-sub _any2index
+method _any2index
 {
-   my $self = shift;
-
    if( ref $_[0] ) {
       my $child = shift;
-      my $index = firstidx { $_ == $child } $self->children;
-      return $index if defined $index;
+      $_children[$_] == $child and return $_ for 0 .. $#_children;
       croak "Unable to find child $child";
    }
    else {
       my $index = shift;
-      return $index if $index >= 0 and $index < $self->children;
+      return $index if $index >= 0 and $index < scalar @_children;
       croak "Index $index out of bounds";
    }
 }
 
-=head2 %opts = $widget->child_opts( $child_or_index )
+=head2 child_opts
+
+   %opts = $widget->child_opts( $child_or_index )
 
 Returns the options currently set for the given child, specified either by
 reference or by index.
 
 =cut
 
-sub child_opts
+method child_opts
 {
-   my $self = shift;
-   my $child = ref $_[0] ? shift : $self->{children}[shift];
+   my $child = ref $_[0] ? shift : $_children[shift];
 
    return unless $child;
 
    return $self->SUPER::child_opts( $child );
 }
 
-=head2 $widget->set_child( $index, $child )
+=head2 set_child
+
+   $widget->set_child( $index, $child )
 
 Replaces the child widget at the given index with the given new one;
 preserving any options that are set on it.
 
 =cut
 
-sub set_child
+method set_child
 {
-   my $self = shift;
    my ( $index, $child ) = @_;
 
-   my $old_child = $self->{children}[$index];
+   my $old_child = $_children[$index];
 
    my %opts;
    if( $old_child ) {
       %opts = $self->child_opts( $old_child );
 
-      local $self->{suppress_redistribute} = 1;
+      dynamically $_suppress_redistribute = 1;
       $self->SUPER::remove( $old_child );
    }
 
-   $self->{children}[$index] = $child;
+   $_children[$index] = $child;
 
    $self->SUPER::add( $child, %opts );
 }
 
-=head2 $widget->set_child_opts( $child_or_index, %newopts )
+=head2 set_child_opts
+
+   $widget->set_child_opts( $child_or_index, %newopts )
 
 Sets new options on the given child, specified either by reference or by
 index. Any options whose value is given as C<undef> are deleted.
 
 =cut
 
-sub set_child_opts
+method set_child_opts
 {
-   my $self = shift;
-   my $child = ref $_[0] ? shift : $self->{children}[shift];
+   my $child = ref $_[0] ? shift : $_children[shift];
 
    return unless $child;
 
    return $self->SUPER::set_child_opts( $child, @_ );
 }
 
-sub render_to_rb
+method render_to_rb
 {
-   my $self = shift;
    my ( $rb, $rect ) = @_;
 
    $rb->eraserect( $rect );
 }
 
-=head2 $widget->add( $child, %opts )
+=head2 add
 
-Adds the widget as a new child of this one, with the given options
+   $widget->add( $child, %opts )
+
+Adds the widget as a new child of this one, with the given options.
+
+This method returns the container widget instance itself making it suitable to
+use as a chaining mutator; e.g.
+
+   my $container = Tickit::Widget::LinearBox->new( ... )
+      ->add( Tickit::Widget::Static->new( ... ) )
+      ->add( Tickit::Widget::Static->new( ... ) );
 
 =cut
 
-sub add
+method add
 {
-   my $self = shift;
    my ( $child, %opts ) = @_;
 
-   push @{ $self->{children} }, $child;
+   push @_children, $child;
 
    $self->SUPER::add( $child,
       expand     => $opts{expand} || 0,
       force_size => $opts{force_size},
    );
+
+   return $self;
 }
 
-=head2 $widget->remove( $child_or_index )
+=head2 add_children
+
+   $widget->add_children( @children )
+
+Adds each of the given widgets as a new child of this one. Each element of the
+list should either be a widget object reference directly, or an unblessed hash
+reference containing additional options. (See
+L<Tickit::Widget/split_widget_opts>).
+
+This method returns the container widget instance itself making it suitable to
+use as a chaining mutator.
+
+=cut
+
+sub add_children
+{
+   my $self = shift;
+
+   foreach my $arg ( @_ ) {
+      $self->add( Tickit::Widget::split_widget_opts $arg );
+   }
+
+   return $self;
+}
+
+=head2 remove
+
+   $widget->remove( $child_or_index )
 
 Removes the given child widget if present, by reference or index
 
 =cut
 
-sub remove
+method remove
 {
-   my $self = shift;
    my $index = $self->_any2index( shift );
 
-   my ( $child ) = splice @{ $self->{children} }, $index, 1, ();
+   my ( $child ) = splice @_children, $index, 1, ();
 
    $self->SUPER::remove( $child ) if $child;
 }
 
-sub reshape
+method reshape
 {
-   my $self = shift;
-   $self->{suppress_redistribute} and return;
+   $_suppress_redistribute and return;
 
    my $window = $self->window;
 
