@@ -1,86 +1,61 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2013-2020 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2013-2021 -- leonerd@leonerd.org.uk
 
-package Tickit::Widget::LinearSplit;
+use Object::Pad 0.09;
+
+package Tickit::Widget::LinearSplit 0.32;
+class Tickit::Widget::LinearSplit
+   extends Tickit::ContainerWidget;
 
-use strict;
-use warnings;
-use base qw( Tickit::ContainerWidget );
 use Tickit::Window 0.32; # needs drag_start
 
-our $VERSION = '0.30';
-
 use Carp;
 
-sub new
+has $_split_fraction = 0.5;
+has $_A_child :reader;
+has $_B_child :reader;
+
+has $_split_at;  method _split_at  { $_split_at }
+has $_split_len; method _split_len { $_split_len }
+
+method set_A_child
 {
-   my $class = shift;
-   my %args = @_;
-
-   my $self = $class->SUPER::new( %args );
-
-   $self->{split_fraction} = 0.5;
-
-   return $self;
-}
-
-sub A_child
-{
-   my $self = shift;
-   $self->{A_child};
-}
-
-sub set_A_child
-{
-   my $self = shift;
    my ( $child ) = @_;
-   $self->remove( $self->{A_child} ) if $self->{A_child};
-   $self->add( $self->{A_child} = $child );
+   $self->remove( $_A_child ) if $_A_child;
+   $self->add( $_A_child = $child );
    return $self;
 }
 
-sub B_child
+method set_B_child
 {
-   my $self = shift;
-   $self->{B_child};
-}
-
-sub set_B_child
-{
-   my $self = shift;
    my ( $child ) = @_;
-   $self->remove( $self->{B_child} ) if $self->{B_child};
-   $self->add( $self->{B_child} = $child );
+   $self->remove( $_B_child ) if $_B_child;
+   $self->add( $_B_child = $child );
    return $self;
 }
 
-sub children
+method children
 {
-   my $self = shift;
-   return grep { defined } $self->{A_child}, $self->{B_child};
+   return grep { defined } $_A_child, $_B_child;
 }
 
-sub child_resized
+method child_resized
 {
-   my $self = shift;
    # TODO: should handle minimums at least
 }
 
 ## This should come from ContainerWidget
-sub children_changed
+method children_changed
 {
-   my $self = shift;
-
    $self->reshape if $self->window;
    $self->resized;
 }
 ##
 
-sub reshape
+method reshape
 {
-   my $self = shift;
    my $win = $self->window or return;
 
    my $spacing = $self->get_style_values( "spacing" );
@@ -88,14 +63,14 @@ sub reshape
    my $method = $self->VALUE_METHOD;
 
    my $quota = $win->$method - $spacing;
-   my $want_split_at = int( $quota * $self->{split_fraction} + 0.5 ); # round to nearest
+   my $want_split_at = int( $quota * $_split_fraction + 0.5 ); # round to nearest
 
    # Enforce child minimum sizes
-   if( my $child = $self->{B_child} ) {
+   if( my $child = $_B_child ) {
       my $max = $quota - $child->$method;
       $want_split_at = $max if $want_split_at > $max;
    }
-   if( my $child = $self->{A_child} ) {
+   if( my $child = $_A_child ) {
       my $min = $child->$method;
       $want_split_at = $min if $want_split_at < $min;
    }
@@ -106,7 +81,7 @@ sub reshape
    my @A_geom = $self->_make_child_geom( 0,                   $A_value );
    my @B_geom = $self->_make_child_geom( $A_value + $spacing, $B_value );
 
-   if( my $child = $self->{A_child} ) {
+   if( my $child = $_A_child ) {
       if( $A_value > 0 ) {
          if( my $childwin = $child->window ) {
             $childwin->change_geometry( @A_geom );
@@ -120,7 +95,7 @@ sub reshape
       }
    }
 
-   if( my $child = $self->{B_child} ) {
+   if( my $child = $_B_child ) {
       if( $B_value > 0 ) {
          if( my $childwin = $child->window ) {
             $childwin->change_geometry( @B_geom );
@@ -134,41 +109,42 @@ sub reshape
       }
    }
 
-   $self->{split_at}  = $A_value;
-   $self->{split_len} = $spacing;
+   $_split_at  = $A_value;
+   $_split_len = $spacing;
 }
 
-sub _on_mouse
+has $_drag_mouse_offset;
+
+method _on_mouse
 {
-   my $self = shift;
    my ( $ev, $val ) = @_;
 
-   my $val0 = $val - $self->{split_at};
-   my $in_split = $val0 >= 0 && $val0 < $self->{split_len};
+   my $val0 = $val - $_split_at;
+   my $in_split = $val0 >= 0 && $val0 < $_split_len;
 
    if( $ev eq "press" ) {
       $self->set_style_tag( active => 1 ) if $in_split;
    }
    elsif( $ev eq "drag_start" ) {
       return unless $in_split;
-      $self->{drag_mouse_offset} = $val0;
+      $_drag_mouse_offset = $val0;
       return 1;
    }
    elsif( $ev eq "drag" ) {
-      return unless defined $self->{drag_mouse_offset};
+      return unless defined $_drag_mouse_offset;
 
       my $method = $self->VALUE_METHOD;
 
-      my $quota = $self->window->$method - $self->{split_len};
+      my $quota = $self->window->$method - $_split_len;
 
-      my $want_split_at = $val - $self->{drag_mouse_offset};
-      $self->{split_fraction} = $want_split_at / $quota;
+      my $want_split_at = $val - $_drag_mouse_offset;
+      $_split_fraction = $want_split_at / $quota;
 
       $self->reshape;
       $self->redraw;
    }
    elsif( $ev eq "drag_drop" ) {
-      undef $self->{drag_mouse_offset};
+      undef $_drag_mouse_offset;
    }
    elsif( $ev eq "release" ) {
       $self->set_style_tag( active => 0 ) if $in_split;
