@@ -1,19 +1,18 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2013 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2013-2021 -- leonerd@leonerd.org.uk
 #  Original render code by Tom Molesworth
 
-package Tickit::Widget::Placegrid;
+use Object::Pad 0.41;  # :param
+
+package Tickit::Widget::Placegrid 0.32;
+class Tickit::Widget::Placegrid
+   extends Tickit::Widget;
 
-use strict;
-use warnings;
-use base qw( Tickit::Widget );
 use Tickit::Style;
 use Tickit::RenderBuffer qw( LINE_SINGLE LINE_THICK );
 
-our $VERSION = '0.30';
-
 use Tickit::Utils qw( textwidth );
 
 =head1 NAME
@@ -70,23 +69,13 @@ dimensions in the centre of the grid.
 
 =cut
 
-sub new
+has $_title :param = undef;
+
+method lines { 1 }
+method cols  { 1 }
+
+method render_to_rb
 {
-   my $class = shift;
-   my %args = @_;
-   my $self = $class->SUPER::new( %args );
-
-   $self->{title} = $args{title};
-
-   return $self;
-}
-
-sub lines { 1 }
-sub cols  { 1 }
-
-sub render_to_rb
-{
-   my $self = shift;
    my ( $rb, $rect ) = @_;
 
    $rb->clear($self->pen);
@@ -103,7 +92,7 @@ sub render_to_rb
 
    $rb->setpen($self->pen);
 
-   if(defined(my $title = $self->{title})) {
+   if(defined(my $title = $_title)) {
       $rb->text_at(($h / 2) - 1, (1 + $w - textwidth($title)) / 2, $title);
    }
 
