@@ -1,18 +1,17 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2013 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2013-2021 -- leonerd@leonerd.org.uk
 
-package Tickit::Widget::VSplit;
+use Object::Pad 0.27;
+
+package Tickit::Widget::VSplit 0.32;
+class Tickit::Widget::VSplit
+   extends Tickit::Widget::LinearSplit;
 
-use strict;
-use warnings;
-use base qw( Tickit::Widget::LinearSplit );
 use Tickit::Style;
 use Tickit::RenderBuffer qw( LINE_SINGLE CAP_BOTH );
 
-our $VERSION = '0.28';
-
 use Carp;
 
 use List::Util qw( sum max );
@@ -23,16 +22,15 @@ C<Tickit::Widget::VSplit> - an adjustable vertical split between two widgets
 
 =head1 SYNOPSIS
 
- use Tickit;
- use Tickit::Widget::VSplit;
- use Tickit::Widget::Static;
+   use Tickit;
+   use Tickit::Widget::VSplit;
+   use Tickit::Widget::Static;
 
- my $vsplit = Tickit::Widget::VSplit->new(
-    left_child  => Tickit::Widget::Static->new( text => "Text above" ),
-    right_child => Tickit::Widget::Static->new( text => "Text below" ),
- );
+   my $vsplit = Tickit::Widget::VSplit->new
+      ->set_left_child ( Tickit::Widget::Static->new( text => "Text above" ) ),
+      ->set_right_child( Tickit::Widget::Static->new( text => "Text below" ) );
 
- Tickit->new( root => $vsplit )->run;
+   Tickit->new( root => $vsplit )->run;
 
 =head1 DESCRIPTION
 
@@ -93,54 +91,40 @@ use constant VALUE_METHOD => "cols";
 
 =head1 CONSTRUCTOR
 
-=head2 $vsplit = Tickit::Widget::VSplit->new( %args )
+=head2 new
+
+   $vsplit = Tickit::Widget::VSplit->new( %args )
 
 Constructs a new C<Tickit::Widget::VSplit> object.
 
-Takes the following named arguments
-
-=over 8
-
-=item left_child => WIDGET
-
-=item right_child => WIDGET
-
-Child widgets to use
-
-=back
-
 =cut
 
-sub new
+BUILD
 {
-   my $class = shift;
    my %args = @_;
 
-   my $self = $class->SUPER::new( %args );
+   croak "The 'left_child' constructor argument to ${\ref $self} is no longer recognised; use ->set_left_child instead"
+      if $args{left_child};
 
-   $self->set_left_child ( $args{left_child}  ) if $args{left_child};
-   $self->set_right_child( $args{right_child} ) if $args{right_child};
-
-   return $self;
+   croak "The 'right_child' constructor argument to ${\ref $self} is no longer recognised; use ->set_right_child instead"
+      if $args{right_child};
 }
 
-sub lines
+method lines
 {
-   my $self = shift;
    return max(
-      $self->{A_child} ? $self->{A_child}->requested_lines : 1,
-      $self->{B_child} ? $self->{B_child}->requested_lines : 1,
+      $self->left_child  ? $self->left_child->requested_lines  : 1,
+      $self->right_child ? $self->right_child->requested_lines : 1,
    );
 }
 
-sub cols
+method cols
 {
-   my $self = shift;
    my $spacing = $self->get_style_values( "spacing" );
    return sum(
-      $self->{A_child} ? $self->{A_child}->requested_cols : 1,
+      $self->left_child  ? $self->left_child->requested_cols  : 1,
       $spacing,
-      $self->{B_child} ? $self->{B_child}->requested_cols : 1,
+      $self->right_child ? $self->right_child->requested_cols : 1,
    );
 }
 
@@ -148,9 +132,13 @@ sub cols
 
 =cut
 
-=head2 $child = $hsplit->left_child
+=head2 left_child
 
-=head2 $vsplit->set_left_child( $child )
+=head2 set_left_child
+
+   $child = $hsplit->left_child
+
+   $vsplit->set_left_child( $child )
 
 Accessor for the child widget used in the left half of the display.
 
@@ -159,50 +147,59 @@ Accessor for the child widget used in the left half of the display.
 *left_child     = __PACKAGE__->can( "A_child" );
 *set_left_child = __PACKAGE__->can( "set_A_child" );
 
-=head2 $child = $hsplit->right_child
+=head2 right_child
 
-=head2 $vsplit->set_right_child( $child )
+=head2 set_right_child
+
+   $child = $hsplit->right_child
+
+   $vsplit->set_right_child( $child )
 
 Accessor for the child widget used in the right half of the display.
 
+These mutators returning the container widget itself making them suitable to
+use as chaining mutators; e.g.
+
+   my $container = Tickit::Widget::VSplit->new( ... )
+      ->set_left_child ( Tickit::Widget::Box->new ... )
+      ->set_right_child( Tickit::Widget::Box->new ... );
+
 =cut
 
 *right_child     = __PACKAGE__->can( "B_child" );
 *set_right_child = __PACKAGE__->can( "set_B_child" );
 
-sub _make_child_geom
+method _make_child_geom
 {
-   my $self = shift;
    my ( $start, $len ) = @_;
    return ( 0, $start, $self->window->lines, $len );
 }
 
-sub render_to_rb
+method render_to_rb
 {
-   my $self = shift;
    my ( $rb, $rect ) = @_;
 
-   my $split_len = $self->{split_len};
+   my $split_len = $self->_split_len;
+   my $split_at  = $self->_split_at;
 
    my $lines = $self->window->lines;
 
    $rb->setpen( $self->get_style_pen( "split" ) );
 
-   $rb->vline_at( 0, $lines-1, $self->{split_at}, LINE_SINGLE, undef, CAP_BOTH );
+   $rb->vline_at( 0, $lines-1, $split_at, LINE_SINGLE, undef, CAP_BOTH );
 
    if( $split_len > 2 ) {
       foreach my $line ( $rect->linerange ) {
-         $rb->erase_at( $line, $self->{split_at} + 1, $split_len - 2 );
+         $rb->erase_at( $line, $split_at + 1, $split_len - 2 );
       }
    }
    if( $split_len > 1 ) {
-      $rb->vline_at( 0, $lines-1, $self->{split_at} + $split_len - 1, LINE_SINGLE, undef, CAP_BOTH );
+      $rb->vline_at( 0, $lines-1, $split_at + $split_len - 1, LINE_SINGLE, undef, CAP_BOTH );
    }
 }
 
-sub on_mouse
+method on_mouse
 {
-   my $self = shift;
    my ( $args ) = @_;
 
    if( $args->type ne "wheel" and $args->button == 1 ) {
